import { Form, Input, Button, Spin, Row, Col } from 'antd';
import { useEffect, useMemo, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import HttpService from '../../../services/httpService';
import { useQuery, useModel } from '../../../hooks';
import { commonRules } from '../../../constants/rules';
import { Select } from "../../../components";

const endpoints = {
  municipio: "municipio",
};

const MunicipioDetalle = () => {
  const [form] = Form.useForm();
  const navigate = useNavigate();
  const [loading, setLoading] = useState(false);
  const query = useQuery();
  const id = query.get("id");
  const [estado, setEstado] = useState("");
  const [timer, setTimer] = useState(null);

  const extraParams = useMemo(() => ({
    idMunicipio: id,
    idEstado: id,
  }), [id]);

  const estadoExtraParams = useMemo(() => {
    let params = {};
    if (estado !== "") {
      params.nombre = estado;
    }
    return params;
  }, [estado]);

  const requestParams = useMemo(() => ({
    name: endpoints.municipio,
    id: id,
    extraParams: extraParams,
    expand: 'estado',
  }), [id, extraParams]);

  const { model, modelLoading } = useModel(requestParams);

  useEffect(() => {
    if (model) {
      form.setFieldsValue({
        ...model,
        idEstado: model.estado?.id,
      });
    }
  }, [form, model]);

  const onFinish = async (values) => {
    try {
      setLoading(true);
      const body = { ...values, id };
      const res = await HttpService.post(`${endpoints.municipio}/guardar`, body);
      if (res?.status === 200) {
        navigate('/administracion/catalogos/municipios');
      }
    } catch (error) {
      console.log(error);
    } finally {
      setLoading(false);
    }
  };

  if (modelLoading) {
    return <Spin size="large" style={{ display: "block", margin: "auto", marginTop: "50px" }} />;
  }

  const onSearch = (value) => {
    clearTimeout(timer);
    const newTimer = setTimeout(() => {
      setEstado(value);
    }, 300);
    setTimer(newTimer);
  };

  return (
    <Form
      layout="vertical"
      name="basic"
      form={form}
      onFinish={onFinish}
      onFinishFailed={() => {}}
    >
      <Row gutter={16}>
        <Col span={24}>
          <h2>Información del Municipio</h2>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Nombre"
            name="nombre"
            rules={[commonRules.requerido]}
          >
            <Input />
          </Form.Item>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Estado"
            name="idEstado"
            rules={[commonRules.requerido]}
          >
            <Select
              modelsParams={{ name: 'estado', ordenar: 'nombre' }}
              labelProp="nombre"
              valueProp="id"
              append={[model?.estado]}
              onSearch={onSearch}
              extraParams={estadoExtraParams}
            />
          </Form.Item>
        </Col>
        <Col span={24}>
          <Form.Item>
            <Button
              type="primary"
              htmlType="submit"
              style={{ marginTop: "30px" }}
              loading={loading}
            >
              Guardar
            </Button>
          </Form.Item>
        </Col>
      </Row>
    </Form>
  );
};

export default MunicipioDetalle;
